//require the needed files
var http = require('http');
var io = require('socket.io');
var users = require('./helpers/users');
var messages = require('./helpers/messages');

//init clients object. here we will save the user details on the socket id key
var clients = {};
//init sockets object. here we will save the socket object on the user id
var sockets = {};

//create the server
var server = http.createServer(function(request, response) {
	request.on('end', function() {
		response.writeHead(200, {'Content-Type': 'text/html'});
		response.end();
	});
}).listen([PORT]);

var conn = io.listen(server);

//socket handlers
conn.sockets.on('connection', function(socket) {
	clients[socket.id] = {};

	//register event
	socket.on('register', function(username, password) {
		//check if username is unique
		var result = users.get_user_by_username(username);
		if (0 < result.length) {
			socket.emit('register_user_response', 'Username already used', 0);
			return;
		}
		
		//check if username is valid
		if (!users.check_username(username)) {
			socket.emit('register_user_response', 'Username not valid', 0);
			return;
		}
		
		//if unique, save the new user and send success message
		users.save_user(username, password);
		socket.emit('register_user_response', 'Account created', 1);
	});
	
	//login event
	socket.on('login', function(username, password) {
		//search user by username
		var result = users.get_user_by_username(username);
		if (0 == result.length) {
			socket.emit('login_user_response', 'Invalid credentials', 0);
			return;
		}
		
		//if user found, check if the password matches
		if (!users.crypt.checkPassword(password, result[0].user_password)) {
			socket.emit('login_user_response', 'Invalid credentials', 0);
			return;
		}
		
		//if password matches, add user details in clients object
		clients[socket.id] = {
			user_username: result[0].user_username,
			user_id: result[0].user_id
		};
		//add socket object in sockets object
		sockets[result[0].user_id] = socket;
		
		//send success message
		socket.emit('login_user_response', result[0].user_id, 1, result[0].user_username);
		//notify other users that this user is now online
		socket.broadcast.emit('users_list_add', result[0].user_username, result[0].user_id);
	});
	
	//auto login event
	socket.on('auto_login', function(user_id) {
		//search user by id
		var result = users.get_user_by_id(user_id);
		
		if (0 == result.length) {
			socket.emit('auto_login_response', 0);
			return;
		}
		
		//if user found, add user details in clients object
		clients[socket.id] = {
			user_username: result[0].user_username,
			user_id: result[0].user_id
		};
		sockets[result[0].user_id] = socket;
		
		//send success message
		socket.emit('auto_login_response', 1);
		//notify other users that this user is no online
		socket.broadcast.emit('users_list_add', result[0].user_username, result[0].user_id);
	});
	
	//get users event
	socket.on('get_users', function() {
		//respond with a json of all the connected clients
		socket.emit('users_list_refresh', JSON.stringify(clients));
	});
	
	//message sent event
	socket.on('message_sent', function(message_to, message_content) {
		//get user id of sender
		message_from = clients[socket.id].user_id;
		//save the message to db and get the id
		var message_id = messages.save_message(message_from, message_to, message_content);

		//if there is a socket to send to
		if (sockets[message_to]) {
			//send the message
			sockets[message_to].emit('message_receive', message_from, message_content, message_id);
		}
	});
	
	//message read request
	socket.on('messages_read', function(messages_ids) {
		//parse the id array of the messages to be marked as read
		var arr = JSON.parse(messages_ids);
		
		//mark this messages as read
		messages.read_messages(arr);
	});
	
	//get some messages of the conversation
	socket.on('get_conv_messages', function(message_with, begin_index, no_elements) {
		//get user id of the user that makes the request
		var user_id = clients[socket.id].user_id;
		
		//get the required messages
		var conv_messages = messages.get_conv_messages(user_id, message_with, begin_index, no_elements);
		
		//send the messages
		socket.emit('get_conv_messages_resp', JSON.stringify(conv_messages));
	});
	
	//disconnect event
	socket.on('disconnect', function() {
		//if there is a user with this socket id
		if (clients[socket.id]) {
			//notify other users that the user is now offline
			socket.broadcast.emit('users_list_remove', clients[socket.id].user_id);
			//remove the socket from sockets object
			delete sockets[clients[socket.id].user_id];
		}
		//remove the user details from clients object
		delete clients[socket.id];
	});
	
	//logout event
	socket.on('logout', function() {
		//if there is a user with this socket id
		if (clients[socket.id]) {
			//notify other users that the user is now offline
			socket.broadcast.emit('users_list_remove', clients[socket.id].user_id);
			//remove the socket from sockets object
			delete sockets[clients[socket.id].user_id];
		}
		//remove the user details from clients object
		delete clients[socket.id];
	});
});

console.log('server started');
